import numpy as np
import pandas as pd
import pytest

import cmprsk

from cmprsk import rpy_utils
from cmprsk.rpy_utils import Dtypes


def get_dataframe():
    return pd.DataFrame(dict(a=[1, 2, 3], b=[1.1, 2.2, 3.3], c=['r', 's', 't']))


def test_r_vec():
    np_int_vector = np.array([1, 2, 3])
    r_int_vector = rpy_utils.r_vec(np_int_vector, Dtypes.int)
    assert (np.array(r_int_vector) == np_int_vector).all()

    np_float_vector = np.array([1.1, 2.1, 3.1])
    r_float_vector = rpy_utils.r_vec(np_float_vector, Dtypes.float)
    assert (np.array(r_float_vector) == np_float_vector).all()

    np_bool_vector = np.array([True, True, False])
    r_bool_vector = rpy_utils.r_vec(np_bool_vector, Dtypes.bool)
    assert (np.array(r_bool_vector) == np_bool_vector).all()

    np_str_vector = np.array(['a', 'b', 'c'])
    r_str_vector = rpy_utils.r_vec(np_str_vector, Dtypes.str)
    assert (np.array(r_str_vector) == np_str_vector).all()

    np_mat = np.array([[1, 2], [3, 4]])
    with pytest.raises(ValueError):
        rpy_utils.r_vector(np_mat)

def test_r_matrix():
    np_mat = np.array([[1, 2], [3, 4]])
    r_mat = rpy_utils.r_matrix(np_mat)
    assert (np_mat == np.array(r_mat)).all()
    assert (np.array(r_mat.colnames) == np.array(['x_1', 'x_2'])).all()



import pandas as pd

from pandas.api.types import is_numeric_dtype


def to_categorical(df, columns):
    """ Creates a COPY and replace the text columns integer values (categories)

    Args:
        df: some data frame
        columns: list of TEXT columns to be interperted as categories

    Returns:
        a copy dataframe with TEXT columns replaced with intgers
    """
    _df = df.copy()
    for c in columns:
        _df.loc[:, c] = df[c].astype('category').cat.codes
    return _df


def non_numeric_columns(df):
    """ Returns a listof non numeric columns """
    return [c for c in df.columns if not is_numeric_dtype(df[c])]


def _to_dummies(df, column_name, base=None):
    _df = df.copy()
    _dummies = pd.get_dummies(df[column_name], prefix=column_name).astype('int64')
    if base is not None:
        _dummies = _dummies.drop('%s_%s' % (column_name, base), axis=1)

    _df = _df.drop(column_name, axis=1)
    return pd.concat([_df, _dummies], axis=1)


def as_indicators(df, column_names, bases=None):
    if bases is None:
        bases = [None] * len(column_names)

    _df = df.copy()
    for col, base in zip(column_names, bases):
        _df = _to_dummies(_df, col, base=base)

    return _df



import numpy as np
import pandas as pd
import pytest
import random
import string


import cmprsk.cmprsk as cmprsk

from cmprsk import utils


def get_data():
    np.random.seed(42)
    random.seed(34)
    ftime = np.random.exponential(size=200)
    fstatus = pd.Series(np.random.randint(0, 3, 200))
    x2 = np.random.exponential(size=200)
    x3 = np.random.randn(200)
    x1 = [random.choice(string.ascii_lowercase) for _ in range(200)]
    x4 = np.random.randint(0, 10, 200).astype(int)
    cov = pd.DataFrame(dict(x1=x1, x2=x2, x3=x3, x4=x4))
    return ftime, fstatus, cov


def test_cmprsk():
    ftime, fstatus, cov = get_data()

    with pytest.raises(cmprsk.NonNumericCovariateError):
        cmprsk.crr(ftime, fstatus, cov)

    cov_1 = utils.as_indicators(cov, ['x1', 'x4'], bases=['d', 5])
    crr_res = cmprsk.crr(ftime, fstatus, cov_1)
    report = crr_res.summary
    assert isinstance(crr_res, cmprsk.CrrResult)



import enum

import numpy as np
import pandas as pd
import rpy2 as R

from pandas.api.types import is_numeric_dtype
# from rpy2 import rinterface
from rpy2 import robjects
from rpy2.robjects import r, pandas2ri, numpy2ri
from rpy2.robjects.packages import importr

pandas2ri.activate()
numpy2ri.activate()


class NotImplementedError(Exception):
    pass


class InputError(Exception):
    pass


class Dtypes(enum.Enum):
    int = R.rinterface.INTSXP
    float = R.rinterface.REALSXP
    bool = R.rinterface.LGLSXP
    str = R.rinterface.STRSXP


def r_vec(np_vec, dtype):
    return R.rinterface.SexpVector(np_vec, dtype.value)


def r_vector(np_vector):
    """Convert a numpy vector to an R vector

    Args:
        np_vector (np.array): 1 dimentional array of bool, int or floats

    Returns:
        rpy2.rinterface.SexpVector: R vector of compatible data type
    """

    if np_vector.ndim != 1:
        msg = 'Input niput dimension is %s and MUST be 1' % np_vector.ndim
        raise ValueError(msg)

    d_type = np_vector.dtype
    if np.issubdtype(d_type, np.integer):
        return r_vec(np_vector, Dtypes.int)
    elif np.issubdtype(d_type, np.floating):
        return r_vec(np_vector, Dtypes.float)
    elif np.issubdtype(d_type, np.bool):
        return r_vec(np_vector,Dtypes.bool)
    elif np.issubdtype(d_type, np.str):
        return r_vec(np_vector,Dtypes.string)
    else:
        msg = "Can't convert vectors with dtype %s yet" % d_type
        raise NotImplementedError(msg)


def r_matrix(np_matrix, col_names=None):
    """Convert a numpymatrix to R matrix. If no columns are provided
    it will assign the following ['x_1', 'x_2',... ] as column names
    """

    if np_matrix.ndim != 2:
        msg = 'Input input dimension is %s and MUST be 2' % np_matrix.ndim
        raise ValueError(msg)

    n_row, n_col = np_matrix.shape
    r_mat = robjects.r.matrix(np_matrix, nrow=n_row, ncol=n_col)
    if col_names is None:
        col_names = ['x_%s' %(i+1) for i in range(n_col)]

    r_mat.colnames = robjects.StrVector(col_names)
    return r_mat


def r_dataframe(pd_dataframe):
    return pandas2ri.py2ri(pd_dataframe)


def parse_r_list(r_list):
    return dict(zip(r_list.names, map(np.array, r_list)))



import numpy as np
import pandas as pd

from rpy2.robjects.packages import importr as import_R
from scipy.stats import norm as normal

from . import rpy_utils
from . import utils


r_cmprsk = import_R('cmprsk')


class NonNumericCovariateError(Exception):
    pass


class CrrResult(object):
    """An incomplete parser for the result coming from crr.

    The full result is accessible via CrrResult.raw
    """
    def __init__(self, r_crr_res):
        self.raw = r_crr_res
        self.parsed = self.asdict()
        self._coefficients = self.parsed['coef']
        self._names = list(self.raw[0].names)
        self._covariance = self.parsed['var']
        self._stderr = np.sqrt(np.diag(self._covariance))
        self._z = self._covariance/self._stderr

    def asdict(self):
        return rpy_utils.parse_r_list(self.raw)

    @property
    def names(self):
        return self._names

    @property
    def coefficients(self):
        return self._coefficients

    @property
    def stderr(self):
        return self._stderr

    @property
    def covariance(self):
        return self._covariance

    @property
    def p_values(self):
        _p_vals = [p_value(beta, std) for beta, std in
                    zip(self.coefficients, self.stderr)]
        return np.array(_p_vals)

    def hazard_ratio(self, conf_level=0.95):
        values = np.zeros(len(self.coefficients))
        confidence = np.zeros((len(self.coefficients), 2))
        edges = np.array(normal.interval(conf_level))
        for i, (coef, std) in enumerate(zip(self.coefficients, self.stderr)):
            values[i] = np.exp(coef)
            confidence[i, :] = [np.exp(coef + edges[0] * std),
                                np.exp(coef + edges[1] * std)]
        return values, confidence

    @property
    def summary(self):
        cols = ['names', 'coefficients', 'stderr', 'hazard_ratio',
                'hazard_ratio_2.5%', 'hazard_ratio_97.5%', 'p_values']
        out = pd.DataFrame(columns=cols)
        out['coefficients'] = self.coefficients
        out['stderr'] = self.stderr
        hazard_ratio, confidence = self.hazard_ratio()
        out['hazard_ratio'] = hazard_ratio
        out[['hazard_ratio_2.5%', 'hazard_ratio_97.5%']] = confidence
        out['p_values'] = self.p_values
        out['names'] = self.names
        return out.set_index('names')

def p_value(x, stderr):
    return 2 * (1 - normal.cdf(abs(x/stderr)))


def crr(ftime, fstatus, static_covariates, cengroup=None, failcode=1, cencode=0,
        subset=None, **kwargs):
    """
    Args:
        ftime (np.array or pandas.Series): time to failure
        fstatus (np.array or pandas.Series):
        static_covariates (pd.DataFrame): time independent  covariates. numeric only dataframe

    """
    non_numeric_cols = utils.non_numeric_columns(static_covariates)
    if non_numeric_cols:
        msg = """

        Input dataframe contains non numeric columns: {}.
        Please convert text columns using `rpy_utils.to_categorical` method first""".format(non_numeric_cols)
        raise NonNumericCovariateError(msg)

    if isinstance(ftime, pd.Series):
        ftime = ftime.values
    r_ftime = rpy_utils.r_vector(ftime)

    if isinstance(fstatus, pd.Series):
        fstatus = fstatus.values
    r_fstatus = rpy_utils.r_vector(fstatus)

    r_static_cov = rpy_utils.r_dataframe(static_covariates)

    if cengroup is not None:
        r_cengroup = rpy_utils.r_vector(cengroup)
        kwargs['cengroup'] = r_cengroup

    if subset is not None:
        r_subset = rpy_utils.r_vector(subset)
        kwargs['subset'] = r_subset

    r_crr_result = r_cmprsk.crr(r_ftime, r_fstatus, r_static_cov,
                                failcode=failcode, cencode=cencode, **kwargs)
    return CrrResult(r_crr_result)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cmprsk",
    version="0.0.2",
    author="Omri Treidel",
    author_email="treidel2@gmail.com",
    description="A python wrapper around cmprsk R package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/OmriTreidel/cmprsk",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
    ],
    install_requires=[
          'numpy',
          'pandas',
          'rpy2',
          'scipy',
    ],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    keywords=[
    'competing risks',
    'survival analysis',
    ],
)



import numpy as np
import pandas as pd
import pytest

from cmprsk import utils


def get_dataframe():
    return pd.DataFrame(dict(a=[2, 2, 0, 2, 0],
                             b=[1.1, 2.2, 3.3, 4, 5],
                             c=['r', 's', 't', 't', 'r']))


def test_non_numeric_columns():
    df = get_dataframe()
    assert ['c'] == utils.non_numeric_columns(df)


def test_to_categorical():
    df = get_dataframe()
    df2 = utils.to_categorical(df, ['c'])
    assert (df2['c'].values == np.array([0, 1, 2, 2, 0])).all()


def test_dummies():
    df = get_dataframe()
    _df = utils._to_dummies(df, 'c', base='t')
    expected = df[['a', 'b']]
    expected['c_r'] = [1, 0, 0, 0, 1]
    expected['c_s'] = [0, 1, 0, 0, 0]
    assert (_df.columns == expected.columns).all()
    assert (_df.values == expected.values).all

    _df = utils._to_dummies(df, 'c')
    expected = df[['a', 'b']]
    expected['c_r'] = [1, 0, 0, 0, 1]
    expected['c_s'] = [0, 1, 0, 0, 0]
    expected['c_t'] = [0, 0, 1, 1, 0]
    assert (_df.columns == expected.columns).all()
    assert (_df.values == expected.values).all


def test_as_indicators():
    df = get_dataframe()
    _df = utils.as_indicators(df, ['a', 'c'], bases=[2, 't'])
    expected = df[['b']]
    expected['a_0'] = [0, 0, 1, 0, 1]
    expected['c_r'] = [1, 0, 0, 0, 1]
    expected['c_s'] = [0, 1, 0, 0, 0]
    assert (_df.columns == expected.columns).all()
    assert (_df.values == expected.values).all



import numpy as np
import pandas as pd


from cmprsk import cmprsk

data  = pd.read_csv('tests/test_set.csv')
print(data)

subset = np.random.randint(0, 2, len(data)).astype(bool)

# kwargs = dict(failcode=1, cencode=2)
res = cmprsk.crr(data.time_to_event.values, data.outcome.values,
                 data[['pim', 'pom', 'pum']], failcode=1, cencode=2, subset=subset)


cmprsk.crr(data.time_to_event.values, data.outcome.values, data[['pim', 'pom', 'pum']], failcode=1, cencode=2)
